"use client";
import React, { useEffect, useState, useRef } from "react";
import Header from "@/components/partials/header";
import Sidebar from "@/components/partials/sidebar";
import { cn } from "@/lib/utils";
import { useSidebar, useThemeStore } from "@/store";
import { motion, AnimatePresence } from "framer-motion";
import { useRouter, usePathname } from "next/navigation";
import Footer from "@/components/partials/footer";
import { useMediaQuery } from "@/hooks/use-media-query";
import ThemeCustomize from "@/components/partials/customizer/theme-customizer";
import MobileSidebar from "@/components/partials/sidebar/mobile-sidebar";
import HeaderSearch from "@/components/header-search";
import { useMounted } from "@/hooks/use-mounted";
import LayoutLoader from "@/components/layout-loader";
import toast, { Toaster } from "react-hot-toast";
import { getWithToken, postWithToken, getApiData } from "@/Helper/common";
import { CrossIcon } from "lucide-react";
import { CloseRounded } from "@mui/icons-material";
import { removeUserToken } from 'action/cookies'
import { useTransition } from "react";

import Link from "next/link";
import {
  Dialog,
  DialogClose,
  DialogContent,
  DialogFooter,
  DialogHeader,
  DialogTitle,
  DialogTrigger,
} from "@/components/ui/dialog";
import { Input } from "@/components/ui/input";
import { Button } from "@/components/ui/button";

const INACTIVITY_TIMEOUT = 8 * 60 * 1000;

const MainLayout = ({ children }) => {

  const [isPending, startTransition] = useTransition();

  
  const totalOtpField = 4;
  const otpArray = Array.from({ length: totalOtpField }, () => "");
  const [otp, setOtp] = useState(otpArray);
  const otpFields = Array.from({ length: totalOtpField }, (_, index) => index);
  const inputRefs = useRef([]);
  const [loadingMpin, setLoadingMpin] = useState(false);
  const [isOpen, setIsOpen] = useState(false);
  const [memberWallet, setMemberWallet] = useState('');
  const [memberWalletLoading, setMemberWalletLoading] = useState(false);

  const handleChange = (e, index) => {
    const { value } = e.target;
    if (!isNaN(value) && value.length <= 1) {
      const newOtp = [...otp];
      newOtp[index] = value;
      setOtp(newOtp);
      if (value.length === 1 && index < totalOtpField - 1) {
        inputRefs.current[index + 1].focus();
      }
    }
  };
  const handleKeyDown = (index, event) => {
    if (event.key === "Backspace" && otp[index] === "" && index > 0) {
      setOtp((prevOtp) => {
        const newOtp = [...prevOtp];
        newOtp[index - 1] = "";
        return newOtp;
      });
      inputRefs.current[index - 1].focus();
    } else if (event.key === "ArrowLeft" && index > 0) {
      inputRefs.current[index - 1].focus();
    } else if (event.key === "ArrowRight" && index < numInputs - 1) {
      inputRefs.current[index + 1].focus();
    }
  };
  const isOtpComplete = otp.every((digit) => digit !== "");


  // useEffect(() => {
  //   const handleBeforeUnload = async (event) => {
  //     event.preventDefault();

  //     console.log('sdasdasd');

  //   };

  //   window.addEventListener('beforeunload', handleBeforeUnload);

  //   return () => {
  //     window.removeEventListener('beforeunload', handleBeforeUnload);
  //   };
  // }, []);

  const useNetworkStatus = () => {
    const [isOnline, setIsOnline] = useState(true);

    useEffect(() => {
      // Check the initial network status
      setIsOnline(navigator.onLine);

      // Add event listeners for online and offline events
      const handleOnline = () => setIsOnline(true);
      const handleOffline = () => setIsOnline(false);

      window.addEventListener('online', handleOnline);
      window.addEventListener('offline', handleOffline);

      // Cleanup event listeners on component unmount
      return () => {
        window.removeEventListener('online', handleOnline);
        window.removeEventListener('offline', handleOffline);
      };
    }, []);

    return isOnline;
  };

  const isOnline = useNetworkStatus();

  useEffect(() => {

    if (!isOnline) {

      toast.error('Please check your internet connection and try again.', {
        position: "bottom-center",
        style: { borderRadius: "10px", background: "#333", color: "#fff" },
      });
      // alert('Please check your internet connection and try again.');
    }

  }, [isOnline]);

  const { collapsed, sidebarType, setCollapsed, subMenu, showText, setShowText } = useSidebar();
  const [open, setOpen] = React.useState(false);
  const [loading, setLoading] = React.useState(false);
  const { layout } = useThemeStore();
  const location = usePathname();
  const isMobile = useMediaQuery("(min-width: 768px)");
  const mounted = useMounted();
  const [showMpinPopup, setShowMpinPopup] = useState(false);
  const [mpin, setMpin] = useState("");
  const [userHasMpin, setUserHasMpin] = useState(false);
  const [userKycData, setUserKycData] = useState([]);
  const [dataTheme, setDataTheme] = useState([]);
  const [pageLoading, setPageLoading] = useState(true);

  var apiCall = true;

  useEffect(() => {

    checkUserMpin();
    dataGet();
  }, []);

  const checkUserMpin = async () => {
    try {

      if (apiCall) {

        apiCall = false;

        const response = await getWithToken("get-kyc-details?encType=Yes");
        setUserKycData(response);
        console.log(response);
        setPageLoading(false);
        if (!response.userData.mpin || response.userData.mpin === '') {
          setUserHasMpin(false);
        } else {
          setUserHasMpin(true);
        }
      }
    } catch (error) {
      console.error("Error checking user MPIN:", error);
      setPageLoading(false);
    }
  };

  useEffect(() => {
    if (!userHasMpin) return;

    const mpinSubmitted = localStorage.getItem("mpinSubmitted");

    // console.log("mpin:", mpinSubmitted);

    if (mpinSubmitted === null || mpinSubmitted === "true") {
      let inactivityTimer;

      const resetInactivityTimer = () => {
        clearTimeout(inactivityTimer);
        inactivityTimer = setTimeout(() => {
          localStorage.setItem("mpinSubmitted", false);
          setShowMpinPopup(true);
        }, INACTIVITY_TIMEOUT);
      };

      const handleUserActivity = () => {
        resetInactivityTimer();
      };

      window.addEventListener("mousemove", handleUserActivity);
      window.addEventListener("keydown", handleUserActivity);
      window.addEventListener("click", handleUserActivity);
      window.addEventListener("scroll", handleUserActivity);

      resetInactivityTimer();

      return () => {
        window.removeEventListener("mousemove", handleUserActivity);
        window.removeEventListener("keydown", handleUserActivity);
        window.removeEventListener("click", handleUserActivity);
        window.removeEventListener("scroll", handleUserActivity);
        clearTimeout(inactivityTimer);
      };
    } else {
      setShowMpinPopup(true);
    }
  }, [showMpinPopup, userHasMpin]);

  async function handleMpinSubmit(e) {

    e.preventDefault();
    toast.dismiss();

    const enteredOtp = otp.join("");

    const apiData = JSON.stringify({ mpin: enteredOtp });

    try {

      setLoading(true);

      const data = await postWithToken("check-mpin", apiData);

      if (data.error == false) {

        // toast.success(data.message, { position: "bottom-center", style: { borderRadius: '10px', background: '#333', color: '#fff' } });

        localStorage.removeItem("mpinSubmitted");
        // setShowMpinPopup(false);
        window.location.reload();
        setMpin("");
        setOtp(Array(otpFields.length).fill(''));

        setLoading(false);


      } else {

        toast.error(data.message, { position: "bottom-center", style: { borderRadius: '10px', background: '#333', color: '#fff' } });
        setLoading(false);
        setMpin("");
        setOtp(Array(otpFields.length).fill(''));
      }


    } catch (errorData) {

      // toast.error(errorData.message, { position: "bottom-center", style: { borderRadius: '10px', background: '#333', color: '#fff' } });
      setLoading(false);
      setMpin("");
      setOtp(Array(otpFields.length).fill(''));
    }

  }

  const logout = async () => {
    typeof window !== 'undefined' ? localStorage.removeItem('token') : null;

    // startTransition(async () => {
    //   await removeUserToken(); // Call the Server Action
    //   toast.success("Logout Successfully", {
    //     position: "bottom-center",
    //     style: { borderRadius: "10px", background: "#333", color: "#fff" },
    //   });

    //   router.replace("/login"); // Navigate to login page
    // });

    removeUserToken();
    window.location.href = '/login';
    toast.success('Logout Successfully', {
      position: "bottom-center",
      style: { borderRadius: "10px", background: "#333", color: "#fff" },
    });
  };

  function hexToHsl(hex) {
    // Remove the leading '#' if it exists
    hex = hex.replace(/^#/, '');

    // Parse the R, G, B values
    const r = parseInt(hex.substring(0, 2), 16);
    const g = parseInt(hex.substring(2, 4), 16);
    const b = parseInt(hex.substring(4, 6), 16);

    // Convert RGB to HSL
    return rgbToHsl(r, g, b);
  }

  function rgbToHsl(r, g, b) {
    // Convert RGB values from 0-255 to 0-1
    r /= 255;
    g /= 255;
    b /= 255;

    // Find min and max values
    const max = Math.max(r, g, b);
    const min = Math.min(r, g, b);
    const delta = max - min;

    // Calculate lightness
    let l = (max + min) / 2;

    let h = 0; // Hue
    let s = 0; // Saturation

    if (delta !== 0) {
      // Calculate saturation
      s = l > 0.5 ? delta / (2 - max - min) : delta / (max + min);

      // Calculate hue
      if (max === r) {
        h = (g - b) / delta + (g < b ? 6 : 0);
      } else if (max === g) {
        h = (b - r) / delta + 2;
      } else {
        h = (r - g) / delta + 4;
      }
      h *= 60;
    }

    return `${Math.round(h)} ${(s * 100).toFixed(1)} ${(l * 100).toFixed(1)}`;
  }

  // Example Usage
  // const hex = '#af04d6';
  // const hsl = hexToHsl(hex);
  // console.log(hsl); // { h: 289, s: 96.7, l: 42.2 }


  var primaryColor = '';
  var secondaryColor = '';
  var colorAllow = true;


  var apiCallGetTheme = true;

  async function dataGet() {

    if (apiCallGetTheme) {
      apiCallGetTheme = false;

      const apiResData = await getApiData("theme-setting/single");
      if (apiResData.error === false) {
        setDataTheme(apiResData);
        // console.log('web_primary_color', apiResData);
        primaryColor = apiResData?.result?.primary_color;
        secondaryColor = apiResData?.result?.secondary_color;

        if (primaryColor) {


          // document.documentElement.style.setProperty('--primary', hexToHsl(primaryColor));
          document.documentElement.style.setProperty('--primary', hexToHsl(secondaryColor));
        }

      }
    }


  }

  if (!mounted && pageLoading) {
    return <LayoutLoader />;
  }


  if (layout === "semibox") {
    return (
      userKycData && userKycData.error == false ?
        <>
          <Header handleOpenSearch={() => setOpen(true)} userKycData={userKycData} />
          <Sidebar userKycData={userKycData} dataTheme={dataTheme} />

          <div
            className={cn("content-wrapper transition-all duration-150 ", {
              "xl:ml-[140px]": collapsed,
              "xl:ml-[272px]": !collapsed,
            })}
          >
            <div
              className={cn(
                "  md:pt-6 pb-[37px] pt-[15px] md:px-6 px-4  page-min-height-semibox ",
                {}
              )}
            >
              <div className="xl:mx-14">
                <LayoutWrapper
                  isMobile={isMobile}
                  setOpen={setOpen}
                  open={open}
                  location={location}
                >
                  {children}
                </LayoutWrapper>
              </div>
            </div>
          </div>
          <Footer />
          <ThemeCustomize />
        </>
        : null
    );
  }



  if (layout === "horizontal") {
    return (
      userKycData && userKycData.error == false ?
        <>
          <Header handleOpenSearch={() => setOpen(true)} userKycData={userKycData} />

          <div className={cn("content-wrapper transition-all duration-150 ")}>
            <div
              className={cn(
                "  md:pt-6 pb-[37px] pt-[15px] md:px-6 px-4  page-min-height-horizontal ",
                {}
              )}
            >
              <LayoutWrapper
                isMobile={isMobile}
                setOpen={setOpen}
                open={open}
                location={location}
              >
                {children}
              </LayoutWrapper>
            </div>
          </div>
          <Footer />
          <ThemeCustomize />
        </>
        : null
    );
  }

  if (sidebarType !== "module") {
    return (

      userKycData && userKycData.error == false ?
        <>

          <style >{`
            
          .marquee{
            background-color: ${dataTheme?.result?.secondary_color} !important;
          }

          .hover\\:bg-primary\\/80:hover {

              background-color: ${dataTheme?.result?.secondary_color || '#846df9'} !important;
              opacity: 0.8;
              
          }

          .data-\\[state\\=checked\\]\\:bg-primary[data-state=checked] {

            background-color: ${dataTheme?.result?.secondary_color || '#846df9'} !important;
          }

          .data-\\[state\\=unchecked\\]\\:bg-default-300[data-state=unchecked] {

            background-color: #9b9b9b !important;
          }

          .bg-primary {
            background-color: ${dataTheme?.result?.secondary_color || '#846df9'} !important;
          }

          .domainTruncate {
            display: inline-block;
            max-width: 20ch !important; /* Limit to 20 characters */
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
          }
          .firstNameTruncate {
            display: inline-block;
            max-width: 20ch; /* Limit to 20 characters */
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
          }
          .lastNameTruncate {
            display: inline-block;
            max-width: 15ch; /* Limit to 20 characters */
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
          }
        `}

          </style>
          <Header handleOpenSearch={() => setOpen(true)} userKycData={userKycData} />
          <Sidebar userKycData={userKycData} dataTheme={dataTheme} />

          <div
            className={cn("content-wrapper transition-all duration-150 ", {
              "xl:ml-[420px]": !collapsed && !showText,
              "xl:ml-[200px]": collapsed && !showText,
              "xl:ml-[290px]": !collapsed && showText,
              "xl:ml-[66px]": collapsed && showText,
            })}
          >
            <div
              className={cn(
                "  md:pt-6 pb-[37px] pt-[15px] md:px-6 px-4  page-min-height ",
                {}
              )}
            >
              <LayoutWrapper
                isMobile={isMobile}
                setOpen={setOpen}
                open={open}
                location={location}
              >
                {children}
              </LayoutWrapper>
            </div>
          </div>
          <Footer />
          <ThemeCustomize />

          {showMpinPopup && userHasMpin ?

            <div className="fixed inset-0 bg-black bg-opacity-50 flex items-center justify-center z-50" style={{ backdropFilter: 'blur(8px)' }}>

              <div className="bg-white rounded-lg p-8 max-w-md w-full">
                <div style={{ textAlign: 'right' }}><Link href="#" onClick={logout}><CloseRounded /></Link></div>
                <h2 className="text-2xl font-bold mb-4 text-center">Enter Your MPIN</h2>
                <p className="mb-6 text-gray-600 text-center">
                  Please enter your MPIN to continue
                </p>
                <form onSubmit={handleMpinSubmit}>
                  <div className="flex flex-wrap lg:gap-1 text-center">
                    {otpFields.map((index) => (
                      <Input
                        key={`otp-code-${index}`}
                        type="password"
                        id={`otp${index}`}
                        name={`otp${index}`}
                        value={otp[index]}
                        onChange={(e) => handleChange(e, index)}
                        onKeyDown={(event) => handleKeyDown(index, event)}
                        maxLength={1}
                        className="w-10 h-10 sm:w-[50px] sm:h-12 rounded border-default-300 text-center text-2xl font-medium text-default-900"
                        ref={(ref) => (inputRefs.current[index] = ref)}
                      />
                    ))}
                  </div>
                  {/* <input
                    type="password"
                    placeholder="Enter MPIN"
                    value={mpin}
                    maxLength={4}
                    minLength={4}
                    onChange={(e) => setMpin(e.target.value.split(" ").join(""))}
                    onInput={(e) => e.target.value = e.target.value.replace(/\D/g, '')}
                    className="w-full px-3 py-2 border border-gray-300 rounded-md mb-4 focus:outline-none focus:ring-2 focus:ring-blue-500"
                  /> */}
                  <div className="mt-6">
                    <Button
                      type="submit"
                      className="w-full"
                      size="lg"
                      disabled={loading}
                    >

                      Submit
                      {loading ? (
                        <svg
                          xmlns="http://www.w3.org/2000/svg"
                          width="2em"
                          height="2em"
                          viewBox="0 0 24 24"
                        >
                          <g
                            fill="none"
                            stroke="currentColor"
                            stroke-linecap="round"
                            stroke-width="2"
                          >
                            <path
                              stroke-dasharray="60"
                              stroke-dashoffset="60"
                              stroke-opacity="0.3"
                              d="M12 3C16.9706 3 21 7.02944 21 12C21 16.9706 16.9706 21 12 21C7.02944 21 3 16.9706 3 12C3 7.02944 7.02944 3 12 3Z"
                            >
                              <animate
                                fill="freeze"
                                attributeName="stroke-dashoffset"
                                dur="1.3s"
                                values="60;0"
                              />
                            </path>
                            <path
                              stroke-dasharray="15"
                              stroke-dashoffset="15"
                              d="M12 3C16.9706 3 21 7.02944 21 12"
                            >
                              <animate
                                fill="freeze"
                                attributeName="stroke-dashoffset"
                                dur="0.3s"
                                values="15;0"
                              />
                              <animateTransform
                                attributeName="transform"
                                dur="1.5s"
                                repeatCount="indefinite"
                                type="rotate"
                                values="0 12 12;360 12 12"
                              />
                            </path>
                          </g>
                        </svg>
                      ) : null}
                    </Button>
                  </div>
                </form>
              </div>
            </div>

            : null
          }
        </>
        : <LayoutLoader />
    );
  }
  return (

    userKycData && userKycData.error == false ?

      <>

        <style >{`
          
          .marquee{
            background-color: #fff !important;
          }
          .domainTruncate {
            display: inline-block;
            max-width: 20ch !important; /* Limit to 20 characters */
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
          }
          .firstNameTruncate {
            display: inline-block;
            max-width: 20ch; /* Limit to 20 characters */
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
          }
          .lastNameTruncate {
            display: inline-block;
            max-width: 15ch; /* Limit to 20 characters */
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
          }
        `}

        </style>
        <Header handleOpenSearch={() => setOpen(true)} userKycData={userKycData} />
        <Sidebar userKycData={userKycData} dataTheme={dataTheme} />

        <div
          className={cn("content-wrapper transition-all duration-150 ", {
            "xl:ml-[420px]": !collapsed && !showText,
            "xl:ml-[200px]": collapsed && !showText,
            "xl:ml-[290px]": !collapsed && showText,
            "xl:ml-[66px]": collapsed && showText,
          })}
        >
          <div
            className={cn(
              "  md:pt-6 md:pb-[37px] pb-28 pt-[15px] md:px-6 px-4  page-min-height ",
              {}
            )}
          >
            <LayoutWrapper
              isMobile={isMobile}
              setOpen={setOpen}
              open={open}
              location={location}
            >
              {children}
            </LayoutWrapper>
          </div>
        </div>
        <Footer handleOpenSearch={() => setOpen(true)} />
        {isMobile && <ThemeCustomize />}


      </>
      : null
  );
};

export default MainLayout;

const LayoutWrapper = ({ children, isMobile, setOpen, open, location }) => {
  return (
    <>
      <motion.div
        key={location}
        initial="pageInitial"
        animate="pageAnimate"
        exit="pageExit"
        variants={{
          pageInitial: {
            opacity: 0,
            y: 50,
          },
          pageAnimate: {
            opacity: 1,
            y: 0,
          },
          pageExit: {
            opacity: 0,
            y: -50,
          },
        }}
        transition={{
          type: "tween",
          ease: "easeInOut",
          duration: 0.5,
        }}
      >
        <main>{children}</main>
      </motion.div>

      <MobileSidebar className="left-[300px]" />
      <HeaderSearch open={open} setOpen={setOpen} />
    </>
  );
};
